"""
Flotorch Strands integration module.

This module provides integration between Flotorch SDK and Strands Agents framework,
allowing you to use Flotorch's LLM gateway with Strands agents.
"""

from .llm import FlotorchStrandsModel

__all__ = ["FlotorchStrandsModel"]

